SystemOrganization addCategory: #'Seaside-Tests-Cache-NG'!

!WAKeyGenerator class methodsFor: '*Seaside-Tests-Cache-NG' stamp: 'pmm 1/31/2014 13:53'!
use: aKeyGenerator during: aNiladicBlock
	| old |
	old := current.
	current := aKeyGenerator.
	^ aNiladicBlock ensure: [
		current := old ]! !

WAKeyGenerator subclass: #WAPrecomputedKeyGenerator
	instanceVariableNames: 'keys current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Cache-NG'!

!WAPrecomputedKeyGenerator class methodsFor: 'instance creation' stamp: 'pmm 1/31/2014 14:01'!
from: start to: end
	^ self keys: (start to: end) asArray! !

!WAPrecomputedKeyGenerator class methodsFor: 'instance creation' stamp: 'pmm 1/31/2014 13:59'!
keys: aSequenceableCollection
	^ self basicNew
		initializeWithKeys: aSequenceableCollection! !

!WAPrecomputedKeyGenerator methodsFor: 'initialization' stamp: 'pmm 1/31/2014 13:59'!
initializeWithKeys: aSequenceableCollection
	self initialize.
	keys := aSequenceableCollection.
	current := 0! !

!WAPrecomputedKeyGenerator methodsFor: 'public' stamp: 'pmm 1/31/2014 13:56'!
keyOfLength: anInteger
	| key |
	key := keys at: current.
	current := current + 1.
	^ key! !

Object subclass: #WACacheNGPerformanceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Cache-NG'!

!WACacheNGPerformanceTest methodsFor: 'testing' stamp: 'pmm 9/10/2013 12:12'!
testObjectStorage
	WACache new! !

TestCase subclass: #WACacheNGTest
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Cache-NG'!

!WACacheNGTest class methodsFor: 'testing' stamp: 'pmm 9/14/2013 10:57'!
isAbstract
	^ self name = #WACacheNGTest! !

!WACacheNGTest methodsFor: 'private' stamp: 'pmm 9/14/2013 10:55'!
createCache
	self subclassResponsibility! !

!WACacheNGTest methodsFor: 'running' stamp: 'pmm 9/14/2013 10:56'!
setUp
	cache := self createCache! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:40'!
testAtPut
	1 to: 100 do: [ :i |
		| absent mapped |
		absent := false.
		cache at: i ifAbsent: [ absent := true ].
		self assert: absent.
		mapped := cache at: i put: i greaseString.
		self assert: mapped = i greaseString.
		mapped := cache at: i ifAbsent: [ self assert: false ].
		self assert: mapped = i greaseString ]! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:40'!
testAtPutOverwrite
	1 to: 100 do: [ :i |
		| absent mapped |
		absent := false.
		cache at: i ifAbsent: [ absent := true ].
		self assert: absent.
		cache at: i put: 'zork'.
		mapped := cache at: i put: i greaseString.
		self assert: mapped = i greaseString.
		mapped := cache at: i ifAbsent: [ self assert: false ].
		self assert: mapped = i greaseString ]! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testCopy
	| copy absent |
	cache at: 1 put: '1'.
	cache at: 2 put: '2'.
	copy := cache copy.
	
	copy remove: '2'.
	copy at: 1 put: '1a'.
	copy at: 3 put: '3'.
	
	"cache:
		1 -> '1'
		2 -> '2'
	copy:
		1 -> '1a'
		3 -> '3'
	"
	
	self assert: (cache at: 1 ifAbsent: [ self assert: false ]) = '1'.
	self assert: (cache at: 2 ifAbsent: [ self assert: false ]) = '2'.
	self assert: (cache keyAtValue: '1' ifAbsent: [ self assert: false ]) = 1.
	self assert: (cache keyAtValue: '2' ifAbsent: [ self assert: false ]) = 2.
	absent := false.
	cache at: 3 ifAbsent: [ absent := true ].
	self assert: absent.
	absent := false.
	cache keyAtValue: '1a' ifAbsent: [ absent := true ].
	self assert: absent.
	cache keyAtValue: '3' ifAbsent: [ absent := true ].
	self assert: absent.
	
	self assert: (copy at: 1 ifAbsent: [ self assert: false ]) = '1a'.
	self assert: (copy at: 3 ifAbsent: [ self assert: false ]) = '3'.
	self assert: (copy keyAtValue: '1a' ifAbsent: [ self assert: false ]) = 1.
	self assert: (copy keyAtValue: '3' ifAbsent: [ self assert: false ]) = 3.
	absent := false.
	copy at: 2 ifAbsent: [ absent := true ].
	self assert: absent.
	absent := false.
	copy keyAtValue: '1' ifAbsent: [ absent := true ].
	self assert: absent.
	copy keyAtValue: '2' ifAbsent: [ absent := true ].
	self assert: absent! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testKeyAtValue
	1 to: 100 do: [ :i |
		| absent key |
		absent := false.
		cache keyAtValue: i greaseString ifAbsent: [ absent := true ].
		self assert: absent.
		cache at: i put: i greaseString.
		key := cache keyAtValue: i greaseString ifAbsent: [ self assert: false ].
		self assert: key = i ]! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 9/14/2013 10:35'!
testKeysAndValuesDo
	| reference readBack |
	reference := Dictionary new.

	1 to: 100 do: [ :i |
		cache at: i put: i greaseString.
		reference at: i put: i greaseString ].
	
	readBack := Dictionary new.
	cache keysAndValuesDo: [ :key :value |
		readBack at: key put: value ].
	
	self assert: readBack = reference! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 9/14/2013 10:35'!
testMultipleMappings
	cache at: 'key1' put: 'value'.
	self should: [ cache at: 'key2' put: 'value' ] raise: Error! !

!WACacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testRemove
	1 to: 100 do: [ :i |
		cache at: i put: i greaseString ].
	
	1 to: 100 do: [ :i |
		| absent |
		self assert: (cache at: i ifAbsent: [ self assert: false ]) = i greaseString.
		cache remove: i greaseString.
		absent := false.
		cache at: i ifAbsent: [ absent := true ].
		self assert: absent ]! !

WACacheNGTest subclass: #WADictionaryCacheNGTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Cache-NG'!

!WADictionaryCacheNGTest methodsFor: 'running' stamp: 'pmm 9/14/2013 10:56'!
createCache
	^ WADictionaryCacheNG new! !

WACacheNGTest subclass: #WAHashCacheNGTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Cache-NG'!

!WAHashCacheNGTest methodsFor: 'running' stamp: 'pmm 9/14/2013 10:56'!
createCache
	^ WAHashCacheNG
		initialSize: 13
		maximumSize: 0
		maximumAbsoluteAge: 0
		maximumRelativeAge: 0
		overflowAction: WAHashCacheNG signalError! !

!WAHashCacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testExpireAbsolute
	"
	max age: 3 sec
	
	put 1
	wait 2 sec
	put 2
	wait 2 sec
	
	1 should be expired
	2 should not be expired"
	| delay absent |
	cache := WAHashCacheNG
		initialSize: 13
		maximumSize: 10
		maximumAbsoluteAge: 3
		maximumRelativeAge: 0
		overflowAction: WAHashCacheNG signalError.
	delay := Delay forSeconds: 2.
		
	cache at: 1 put: '1'.
	delay wait.
	cache at: 2 put: '2'.
	delay wait.
	
	"1 should be gone"
	absent := false.
	cache at: 1 ifAbsent: [ absent := true ].
	self assert: absent.
	
	"2 should still be there"
	self assert: (cache at: 2 ifAbsent: [ self assert: false ]) = '2'! !

!WAHashCacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testExpireRelative
	"
	max age: 3 sec
	
	put 1
	put 2
	wait 2 sec
	access 1
	wait 2 sec
	
	1 should not be expired
	2 should be expired"
	| delay absent |
	cache := WAHashCacheNG
		initialSize: 13
		maximumSize: 10
		maximumAbsoluteAge: 0
		maximumRelativeAge: 3
		overflowAction: WAHashCacheNG signalError.
	delay := Delay forSeconds: 2.
		
	cache at: 1 put: '1'.
	cache at: 2 put: '2'.
	delay wait.	
	cache at: 1 ifAbsent: [ self assert: false ].
	delay wait.
	
	"1 should still be there"
	self assert: (cache at: 1 ifAbsent: [ self assert: false ]) = '1'.
	
	"2 should be gone"
	absent := false.
	cache at: 2 ifAbsent: [ absent := true ].
	self assert: absent! !

!WAHashCacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testRemoveAbsoluteOldest
	cache := WAHashCacheNG
		initialSize: 13
		maximumSize: 10
		maximumAbsoluteAge: 0
		maximumRelativeAge: 0
		overflowAction: WAHashCacheNG removeAbsoluteOldest.
		
	1 to: 20 do: [ :i |
		cache at: i put: i greaseString ].
	
	1 to: 10 do: [ :i |
		| absent |
		absent := false.
		cache at: i ifAbsent: [ absent := true ].
		self assert: absent ].
	
	11 to: 20 do: [ :i |
		| mapped |
		mapped := cache at: i ifAbsent: [ self assert: false ].
		self assert: mapped = i greaseString ]! !

!WAHashCacheNGTest methodsFor: 'testing' stamp: 'pmm 1/25/2014 16:41'!
testRemoveRelativeOldest
	"set maximum size = 10
	add 1, 2, ... 10
	access 1, 2, ... 9 -> 10 has not been accessed for the most time
	add 11
	1, 2, ...9, 11 should be in map"
	| tenMissing |
	cache := WAHashCacheNG
		initialSize: 13
		maximumSize: 10
		maximumAbsoluteAge: 0
		maximumRelativeAge: 0
		overflowAction: WAHashCacheNG removeRelativeOldest.
		
	1 to: 10 do: [ :i |
		cache at: i put: i greaseString ].
	
	"access every but 10"
	1 to: 9 do: [ :i |
		cache at: i ifAbsent: [ self assert: false ] ].
	
	cache at: 11 put: '11'.
	
	1 to: 9 do: [ :i |
		self assert: (cache at: i ifAbsent: [ self assert: false ]) = i greaseString ].
	
	self assert: (cache at: 11 ifAbsent: [ self assert: false ]) = '11'.
	
	tenMissing := false.
	cache at: 10 ifAbsent: [ tenMissing := true ].
	self assert: tenMissing
	! !

!WAHashCacheNGTest methodsFor: 'testing' stamp: 'pmm 9/11/2013 08:53'!
testSignalError
	cache := WAHashCacheNG
		initialSize: 13
		maximumSize: 10
		maximumAbsoluteAge: 0
		maximumRelativeAge: 0
		overflowAction: WAHashCacheNG signalError.
		
	1 to: 10 do: [ :i |
		cache at: i put: i greaseString ].
	
	self should: [ cache at: 11 put: '11' ] raise: WAMaximumNumberOfSessionsExceededError! !
